// Board uMighty1284P R3 oder R4
//  onBoard RTC RV3029 (0x56)
//  onBoard SD-Card Socket (9)
//  angeschlossenes Display (0x3C) OLED SH1106 128x64
// ----------------------------------------------------


#include <Wire.h>    // Library für I2C 
#include "rv3029.h"  // Needed to make the IDE accept our time struct.
#include "U8glib.h"  // Library für das OLED LCD (und auch viele andere Typen)

#define I2C_RTC_ADDRESS (0x56)
#define SD_CardSelectPin 9
// die adress des OLED Displays SH1106 is fest 0x3C daher keine Vorgabe nötig

int oldsecond; // anstelle eines Delays warten auf neue Sekunde

String stringZeit = "";              // Speicher reservieren
String stringDatum = "";             // Speicher reservieren
char datum[17] = "";                 // Zeichenarray für Datum  (das Display arbeutet nicht mit Strings)
char zeit[9] = "";                   // Zeichen für Zeit        (das Display arbeutet nicht mit Strings)

// ---------- Display ----------
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);  // Das Display definieren


int LED1 = 1;

rtc_time_t rtc_time;

void setup(void) {
  Serial.begin(9600);
  Wire.begin();
  //stringZeit = String("00-00-00");
  //stringDatum = String("So. 01 Jan. 2017");

  // ---------- Display Grundeinstellungen
  // flip screen, if required
  // u8g.setRot180(); // Kommentar aufheben wenn die Anzeige auf dem Kopf steht

  // set SPI backup if required
  // u8g.setHardwareBackup(u8g_backup_avr_spi);

  // assign default color value
  if ( u8g.getMode() == U8G_MODE_R3G3B2 )
    u8g.setColorIndex(255);     // white
  else if ( u8g.getMode() == U8G_MODE_GRAY2BIT )
    u8g.setColorIndex(3);         // max intensity
  else if ( u8g.getMode() == U8G_MODE_BW )
    u8g.setColorIndex(1);         // pixel on


  // Setup RTC chip.
  Wire.beginTransmission(I2C_RTC_ADDRESS);
  Wire.write(0x00);  // CONTROL_1 address.
  Wire.write(0x01);  // CONTROL_1: Enable 1 Hz clock source for watch.
  Wire.write(0x00);  // CONTROL_INT: Disable all interrupts.
  Wire.write(0x00);  // CONTROL_INT FLAG: Clear all.
  Wire.write(0x00);  // CONTROL_STATUS: Clear all, including PON.
  Wire.endTransmission();

  // ---------- RTC Einstellen der Zeitwerte  
  //            werden noch nicht in den Chip geschrieben
  rtc_time.seconds = 00;
  rtc_time.minutes = 16;
  rtc_time.hours = 21;
  rtc_time.day_of_week = 02;
  rtc_time.day_of_month = 28;
  rtc_time.month = 02;
  rtc_time.year = 17;

  // mit dem folgenden Befehl werden die Daten in den RTC geschrieben 
  // rtc_set_time(I2C_RTC_ADDRESS,rtc_time);


  pinMode(LED1, OUTPUT);
  digitalWrite(LED1, HIGH);
  
}
void loop(void) {
 

  
  rtc_read_time(I2C_RTC_ADDRESS, rtc_time);

  u8g.firstPage();

  if (oldsecond != rtc_time.seconds) {
    // anstelle eines Delays warten bis sich die Sekunde geändert hat
    digitalWrite(LED1, !digitalRead(LED1));


    do {
      stringZeitAufbauen(rtc_time.hours, rtc_time.minutes, rtc_time.seconds);
      stringDatumAufbauen(rtc_time.day_of_week, rtc_time.day_of_month, rtc_time.month, rtc_time.year);

      stringDatum.toCharArray(datum, 17);  

      stringZeit.toCharArray(zeit, 9);

      drawDateTime(datum, zeit);
    } while (u8g.nextPage() );


    // Serielle Ausgabe
    print_time(rtc_time, false);
    Serial.print(" - ");
    print_date(rtc_time, true);
  }
  oldsecond = rtc_time.seconds;

}
void drawDateTime(char d[17], char t[9]) {
  // graphic commands to redraw the complete screen should be placed here
  
  u8g.setFont(u8g_font_unifont);
  u8g.setFontPosTop();
  u8g.drawStr(0, 1, d);
  u8g.drawHLine(0, 1 + 18, 128);
  u8g.setScale2x2();          // Scale up all draw procedures

  u8g.drawStr(0, 12, t);     // actual display position is (0,24)

  //u8g.drawHLine(0, 12+14, 40);    // All other procedures are also affected
  u8g.undoScale();          // IMPORTANT: Switch back to normal mode
}
uint8_t bcd2bin(uint8_t val) {
  return ((val) & 0x0f) + ((val) >> 4) * 10;
}
uint8_t bin2bcd(uint8_t val) {
  return (((val / 10) << 4) | (val % 10));
}
void rtc_set_time(uint8_t i2c_address, rtc_time_t& rtc_time) {
  Wire.beginTransmission(i2c_address);
  Wire.write(0x08);  // Address of seconds register.
  Wire.write(bin2bcd(rtc_time.seconds));  // Set seconds
  Wire.write(bin2bcd(rtc_time.minutes));  // Set minutes
  Wire.write(bin2bcd(rtc_time.hours));  // Set hours
  Wire.write(bin2bcd(rtc_time.day_of_month));  // Set date
  Wire.write(bin2bcd(rtc_time.day_of_week));  // Set weeek day
  Wire.write(bin2bcd(rtc_time.month));  // Set month.
  Wire.write(bin2bcd(rtc_time.year));  // Set year.
  Wire.endTransmission();
}
void rtc_read_time(uint8_t i2c_address, rtc_time_t& rtc_time) {
  Wire.beginTransmission(i2c_address);
  Wire.write(0x08);  // Address of first time register (seconds).
  Wire.endTransmission();
  // Request the seven date/time bytes.
  Wire.requestFrom(i2c_address, (uint8_t)7); // Typecast required to avoid C++ warning.
  rtc_time.seconds = bcd2bin(Wire.read() & 0x7f);
  rtc_time.minutes = bcd2bin(Wire.read() & 0x7f);
  uint8_t hours = Wire.read(); // Hours need more processing.
  rtc_time.day_of_month = bcd2bin(Wire.read() & 0x3f);
  rtc_time.day_of_week = bcd2bin(Wire.read() & 0x07);
  rtc_time.month = bcd2bin(Wire.read() & 0x1f);
  rtc_time.year = bcd2bin(Wire.read());

  // Convert hours.
  if ((hours & (1 << 6)) != 0)
  {
    // 12h format.
    rtc_time.hours = bcd2bin(hours & 0x1f);
    // Check PM flag.
    if ((hours & (1 << 5)) != 0) rtc_time.hours += 12;
  }
  else rtc_time.hours = bcd2bin(hours & 0x3f);
}
void print_time_value(uint16_t value) {
  if (value <= 9) Serial.print('0');
  Serial.print(value);
}
void print_time(rtc_time_t& rtc_time, boolean newline) {
  print_time_value(rtc_time.hours);
  Serial.print(":");
  print_time_value(rtc_time.minutes);
  Serial.print(":");
  print_time_value(rtc_time.seconds);
  if (newline == true) Serial.println();
}
void print_date(rtc_time_t& rtc_time, boolean newline) {
  print_time_value(rtc_time.day_of_month);
  Serial.print("/");
  print_time_value(rtc_time.month);
  Serial.print("/");
  print_time_value(rtc_time.year + 2000);
  if (newline == true) Serial.println();
}
void stringZeitAufbauen(int stun, int minu, int seku) {
  // stringZeit muss definiert sein
  String stringDatum;
  stringZeit = "";

  if (stun <= 9) {
    stringZeit = " ";
  }
  stringZeit += stun;
  stringZeit += ":";

  if (minu <= 9) {
    stringZeit += "0";
  }
  stringZeit += minu;
  stringZeit += ":";

  if (seku <= 9) {
    stringZeit += "0";
  }
  stringZeit += seku;

}
void stringDatumAufbauen(int wtag, int tag, int monat, int jahr) {
  // stringDatum muss definiert sein

  switch (wtag) {
    case 1: stringDatum = "Mo.";
      break;
    case 2: stringDatum = "Di.";
      break;
    case 3: stringDatum = "Mi.";
      break;
    case 4: stringDatum = "Do.";
      break;
    case 5: stringDatum = "Fr.";
      break;
    case 6: stringDatum = "Sa.";
      break;
    case 7: stringDatum = "So.";
      break;
    default: stringDatum = "   ";
      // default is optional
      break;
  }
  stringDatum += " ";
  if (tag <= 9) {
    stringDatum += " ";
  }
  stringDatum += tag;
  stringDatum += ".";
  switch (monat) {
    case 1: stringDatum += "Jan.";
      break;
    case 2: stringDatum += "Feb.";
      break;
    case 3: stringDatum += "Mrz.";
      break;
    case 4: stringDatum += "Apr.";
      break;
    case 5: stringDatum += "Mai.";
      break;
    case 6: stringDatum += "Jun.";
      break;
    case 7: stringDatum = "Jul.";
      break;
    case 8: stringDatum += "Aug.";
      break;
    case 9: stringDatum += "Sep.";
      break;
    case 10: stringDatum = "Okt.";
      break;
    case 11: stringDatum += "Nov.";
      break;
    case 12: stringDatum += "Dez.";
      break;
    default: stringDatum += "    ";
      // if nothing else matches, do the default
      break;
  }
  stringDatum += " 20";  // Das Jahr ergänzen
  stringDatum += jahr;
}

